funcprot(0);
stacksize('max');

function [v,bv,uv,fe,np] = SetupSquare(z1,z2,xn,yn);
// SetupSquare 矩形領域の問題に必要な設定
// z1 z2 領域の対角頂点、xn yn x方向・y方向の分割数(格子点数=(xn+1)×(yn+1)に注意)
// v 格子点座標、bv 境界格子点番号、uv 未知格子点番号
// fe 三角形有限要素の格子点番号、np 格子点を含む有限要素番号

// v 頂点番号別座標（複素表現）
v = zeros(yn+1,xn+1);
s = min(real([z1,z2])); e = max(real([z1,z2])); d = (e - s)/xn;
v = ones(yn+1,1)*[s:d:e];
s = min(imag([z1,z2])); e = max(imag([z1,z2])); d = (e - s)/yn;
v = v + %i*[s:d:e].'*ones(1,yn+1);
v = matrix(v,(xn+1)*(yn+1),1).';

// bv(j) j番目の境界頂点の頂点番号
bv = [1:yn+1 ...
	matrix([[1:xn-1]*(yn+1)+1;[2:xn]*(yn+1)],1,2*(xn-1)) ...
	xn*(yn+1)+1:(xn+1)*(yn+1)];

// uv(j) j番目の未知頂点(=境界じゃない頂点)の頂点番号
uv = matrix([1:yn-1]'*ones(1,xn-1)+ones(yn-1,1)*([1:xn-1]*(yn+1)+1),1,(xn-1)*(yn-1));

// fe(k,:) k番目の(三角形)有限要素の 3つの頂点の頂点番号
fe = zeros(2*yn*xn,3);
for i = 1:yn
  for j = 1:xn
    l = i+(j-1)*(yn+1);
    k = (2*i-1)+2*(j-1)*yn;
    fe(k,1) = l; fe(k,2) = l + (yn+1); fe(k,3) = l + (yn+2);
    k = k + 1;
    fe(k,1) = l; fe(k,2) = l + (yn+2); fe(k,3) = l +1;
  end
end

// np(k,1) k番目の頂点を含む有限要素の数
// np(k,2〜np(k,1)+1) k番目の頂点を含むを含む np(k,1) 個の有限要素番号
np = zeros((yn+1)*(xn+1),1);
for i = 1:2*yn*xn // i 有限要素番号
  for j = 1:3
    np(fe(i,j),1) = np(fe(i,j),1) + 1;
    np(fe(i,j),np(fe(i,j),1)+1) = i;
  end
end
endfunction

function r = Prod(j,k);
// Prod 頂点 j と k 周りの基底関数による重み付残差(弱形式)

r = 0;
for l = np(j,2:np(j,1)+1);
  for m = np(k,2:np(k,1)+1);
    if (l == m)
      r = r + Prod0(l,j,k);
    end
  end
end

endfunction

function r = iprod(z1,z2);
// iprod (z1,z2) 内積(inner product)
r = real(conj(z1)*z2);
endfunction

function r = xprod(z1,z2);
// iprod z1×z2 外積(cross product)
r = imag(conj(z1)*z2);
endfunction

function f = IsInTriangle(z,l,m,n);
// IsInTriangle 三角形に含まれるか検査
// z 検査点、l m n 三角形の3頂点の頂点番号

//if (xprod(v(m)-v(l),z-v(l))*xprod(z-v(l),v(n)-v(l)) >= 0 &...
//    xprod(v(n)-v(m),z-v(m))*xprod(z-v(m),v(l)-v(m)) >= 0)
if (xprod(v(m)-v(l),z-v(l)) >=0 & xprod(z-v(l),v(n)-v(l)) >= 0 &...
    xprod(v(n)-v(m),z-v(m)) >=0 & xprod(z-v(m),v(l)-v(m)) >= 0)
  f = %T;
else
  f = %F;
end

endfunction

function r = Afunc(z,j,k);
// Afunc 有限要素 j の頂点 k で 1 となる基底関数

select k
case 1 then
  l = fe(j,1);
  m = fe(j,2);
  n = fe(j,3);
case 2 then
  l = fe(j,2);
  m = fe(j,3);
  n = fe(j,1);
case 3 then
  l = fe(j,3);
  m = fe(j,1);
  n = fe(j,2);
else
  m =0; n =0;
end
if (m*n <> 0 & IsInTriangle(z,l,m,n))
  r = (xprod(v(m)-v(l),z-v(l))+xprod(z-v(l),v(n)-v(l)))/...
      xprod(v(m)-v(l),v(n)-v(l));
else
  r = 0;
end

endfunction

function r = Prod0(i,j,k);
// Prod0 頂点 j,k を基点とする基底関数の有限要素 i 上での積分

if ((fe(i,1)-j)*(fe(i,2)-j)*(fe(i,3)-j) == 0 & ...
    (fe(i,1)-k)*(fe(i,2)-k)*(fe(i,3)-k) == 0)
select j
case fe(i,1) then
  j2 = fe(i,2);
  j3 = fe(i,3);
case fe(i,2) then
  j2 = fe(i,3);
  j3 = fe(i,1);
case fe(i,3) then
  j2 = fe(i,1);
  j3 = fe(i,2);
end
select k
case fe(i,1) then
  k2 = fe(i,2);
  k3 = fe(i,3);
case fe(i,2) then
  k2 = fe(i,3);
  k3 = fe(i,1);
case fe(i,3) then
  k2 = fe(i,1);
  k3 = fe(i,2);
end
  r = imag(v(j2)-v(j3))*imag(v(k2)-v(k3))+...
      real(v(j3)-v(j2))*real(v(k3)-v(k2));
else
  r = 0
end

endfunction

function r = Bfunc(z);
if imag(z) == 0;
  r = real(z)*(1-real(z));
else
  r = 0;
end
endfunction

function n = InWhichTriangle(z);
j = 0;
for i = 1:size(fe,1);
  if IsInTriangle(z,fe(i,1),fe(i,2),fe(i,3))
    j = i;
    break
  end
end
n = j;
endfunction

//function r = Ufunc(z);
//r=0
//l = InWhichTriangle(z);
//for i = 1:3
//  r = r + u(fe(l,i))*Afunc(z,l,i);
//end
//endfunction

// メインルーチン
[v,bv,uv,fe,np] = SetupSquare(0,1+%i,12,12);
bn = size(bv,2); // 境界頂点の数
ub = zeros(1,bn); // 境界値
for i = 1:bn
  ub(i) = Bfunc(v(bv(i))); // 境界条件の設定
end

un = size(uv,2); // 未知頂点の数
uu = zeros(1,un); // 未知変数

A = zeros(un,un);
for j = 1:un
  for k = 1:un
    A(k,j) = Prod(uv(k),uv(j));
  end
end

B = zeros(un,1);
for j = 1:bn
  for k = 1:un
    B(k) = B(k) - ub(j)*Prod(uv(k),bv(j));
  end
end
uu = A\B;

u = zeros(v); // 近似関数の頂点における値
for i = 1:size(bv,2)
  u(bv(i)) = ub(i);
end
for i = 1:size(uv,2)
  u(uv(i)) = uu(i);
end

//三角形マップの描画
fec(real(v),imag(v),[[1:size(fe,1)]',fe,ones(size(fe,1),1)],u);